import pathlib

import test.mock.uv
from uv_ci_tools.lib import uv, ver

from ._util import ExecutableContext


def test_parse_tool_list_show_paths():
    with (
        test.mock.uv.set_uv_tool_list_show_paths_output(
            """\
- tool0 (/)
pkg1 v0.1.2 (a)
pkg2 v1.0.0 (b/c)
- tool1 (/d/e/f)
pkg3 v0.0.1 (/g)
- tool2 (/)
- tool3 (h)
- tool0
pkg0 v0 (/g)
pkg0 v0.0.0
"""
        ),
        ExecutableContext.make() as exe_ctx,
    ):
        exe_ctx.add_executable(test.mock.uv)
        assert uv.list_installed_packages() == [
            uv.InstalledPackage('pkg1', ver.Version(0, 1, 2), pathlib.Path('a'), tools=[]),
            uv.InstalledPackage(
                'pkg2',
                ver.Version(1, 0, 0),
                pathlib.Path('b/c'),
                tools=[uv.InstalledTool('tool1', pathlib.Path('/d/e/f'))],
            ),
            uv.InstalledPackage(
                'pkg3',
                ver.Version(0, 0, 1),
                pathlib.Path('/g'),
                tools=[
                    uv.InstalledTool('tool2', pathlib.Path('/')),
                    uv.InstalledTool('tool3', pathlib.Path('h')),
                ],
            ),
        ]



import subprocess

from uv_ci_tools.lib import ci, cli, util, uv

APP = cli.sub_app(__name__)


@APP.default
def pre_compile(*, ci_type: ci.Type = ci.Type.GITLAB, project_name: str | None = None):
    ci_ctx = ci_type.fill_context(ci.PartialContext(project_name=project_name))
    installed_package = uv.get_installed_package(ci_ctx.project_name)
    with util.devnull() as devnull:
        subprocess.run(
            [installed_package.python_executable, '-m', 'compileall', installed_package.path],
            check=True,
            stdout=devnull,
        )



import pytest

from uv_ci_tools.lib import ver


def test_load():
    with pytest.raises(RuntimeError, match='no value'):
        ver.Version.load('')

    with pytest.raises(RuntimeError, match='3 parts'):
        ver.Version.load('1.2.3.4')

    with pytest.raises(RuntimeError, match='3 parts'):
        ver.Version.load('1.2')

    with pytest.raises(RuntimeError, match='integer'):
        ver.Version.load('1.2.a')

    assert ver.Version.load('1.2.3') == ver.Version(1, 2, 3)


def test_dump():
    assert ver.Version.load('1.2.3').dump() == '1.2.3'


def test_major() -> None:
    major = ver.IncrementKind.MAJOR
    assert ver.Version(1, 0, 0).incremented(major) == ver.Version(2, 0, 0)
    assert ver.Version(2, 57, 2).incremented(major) == ver.Version(3, 0, 0)
    assert ver.Version(0, 0, 0).incremented(major) == ver.Version(1, 0, 0)
    assert ver.Version(0, 0, 17).incremented(major) == ver.Version(1, 0, 0)


def test_minor() -> None:
    minor = ver.IncrementKind.MINOR
    assert ver.Version(1, 0, 0).incremented(minor) == ver.Version(1, 1, 0)
    assert ver.Version(1, 12, 0).incremented(minor) == ver.Version(1, 13, 0)
    assert ver.Version(3, 0, 5).incremented(minor) == ver.Version(3, 1, 0)
    assert ver.Version(0, 3, 0).incremented(minor) == ver.Version(0, 4, 0)
    assert ver.Version(0, 0, 2).incremented(minor) == ver.Version(0, 1, 0)


def test_patch() -> None:
    patch = ver.IncrementKind.PATCH
    assert ver.Version(1, 0, 0).incremented(patch) == ver.Version(1, 0, 1)
    assert ver.Version(5, 37, 0).incremented(patch) == ver.Version(5, 37, 1)
    assert ver.Version(0, 0, 0).incremented(patch) == ver.Version(0, 0, 1)
    assert ver.Version(0, 0, 99).incremented(patch) == ver.Version(0, 0, 100)



import os
import pathlib

import pytest

import test.mock
import test.mock.git
import test.mock.uv
import test.projects
from uv_ci_tools.app.bump_version import app
from uv_ci_tools.lib import ci, util, ver

from ._util import ExecutableContext, remove_gitlab_ci_vars


def run_app(
    project_name: str,
    *,
    increment_kind: ver.IncrementKind = ver.IncrementKind.PATCH,
    ci_type: ci.Type = ci.Type.GITLAB,
    disable_uv: bool = False,
    disable_git: bool = False,
    disable_project_name: bool = False,
    project_path: str | None = 'project_path',
    commit_ref_name: str | None = 'commit_ref_name',
    email: str | None = 'email',
    username: str | None = 'username',
    password: str | None = 'password',
    repository_host: str | None = 'repository_host',
    no_uv: bool | None = None,
):
    with (
        util.make_tmp_dir_copy(util.module_path(test.projects) / project_name) as project_dir,
        ExecutableContext.make() as exe_ctx,
    ):
        os.chdir(project_dir)

        if not disable_git:
            exe_ctx.add_executable(test.mock.git)
        if not disable_uv:
            exe_ctx.add_executable(test.mock.uv)

        app.version(
            increment_kind,
            ci_type=ci_type,
            project_name=project_name if not disable_project_name else None,
            project_path=project_path,
            commit_ref_name=commit_ref_name,
            email=email,
            username=username,
            password=password,
            repository_host=repository_host,
            no_uv=no_uv,
        )
        return test.mock.git.get_state(project_dir)


def test_simple():
    git_state = run_app('simple')
    assert git_state == test.mock.git.State(
        user_name='username',
        user_email='email',
        remote_name_url_map={
            'origin': 'https://username:password@repository_host/project_path.git'
        },
        staged_files={},
        commits=[],
        pushed=[
            test.mock.git.Pushed(
                commit=test.mock.git.Commit(
                    message='Bumped version to 0.1.1',
                    files={
                        pathlib.Path(
                            'pyproject.toml'
                        ): '[project]\nname = "simple"\nversion = "0.1.1"\n',
                        pathlib.Path('uv.lock'): '',
                    },
                ),
                repository='origin',
                refspec='HEAD:commit_ref_name',
                option='ci.skip',
            )
        ],
    )


def test_simple_no_uv():
    git_state = run_app('simple', no_uv=True)
    assert git_state == test.mock.git.State(
        user_name='username',
        user_email='email',
        remote_name_url_map={
            'origin': 'https://username:password@repository_host/project_path.git'
        },
        staged_files={},
        commits=[],
        pushed=[
            test.mock.git.Pushed(
                commit=test.mock.git.Commit(
                    message='Bumped version to 0.1.1',
                    files={
                        pathlib.Path(
                            'pyproject.toml'
                        ): '[project]\nname = "simple"\nversion = "0.1.1"\n',
                        pathlib.Path('uv.lock'): '',
                    },
                ),
                repository='origin',
                refspec='HEAD:commit_ref_name',
                option='ci.skip',
            )
        ],
    )


def test_with_empty_lock():
    git_state = run_app('with_empty_lock', no_uv=True)
    assert git_state == test.mock.git.State(
        user_name='username',
        user_email='email',
        remote_name_url_map={
            'origin': 'https://username:password@repository_host/project_path.git'
        },
        staged_files={},
        commits=[],
        pushed=[
            test.mock.git.Pushed(
                commit=test.mock.git.Commit(
                    message='Bumped version to 1.0.1',
                    files={
                        pathlib.Path(
                            'pyproject.toml'
                        ): '[project]\nname = "with_empty_lock"\nversion = "1.0.1"\n',
                        pathlib.Path('uv.lock'): '',
                    },
                ),
                repository='origin',
                refspec='HEAD:commit_ref_name',
                option='ci.skip',
            )
        ],
    )


def test_with_lock():
    git_state = run_app('with_lock', no_uv=True)
    assert git_state == test.mock.git.State(
        user_name='username',
        user_email='email',
        remote_name_url_map={
            'origin': 'https://username:password@repository_host/project_path.git'
        },
        staged_files={},
        commits=[],
        pushed=[
            test.mock.git.Pushed(
                commit=test.mock.git.Commit(
                    message='Bumped version to 1.2.4',
                    files={
                        pathlib.Path(
                            'pyproject.toml'
                        ): '[project]\nname = "with_lock"\nversion = "1.2.4"\n',
                        pathlib.Path('uv.lock'): '\n'.join(  # noqa: FLY002
                            [
                                '[[package]]',
                                'name = "with_lock"',
                                'version = "1.2.4"',
                                'source = { editable = "." }',
                                '',
                            ]
                        ),
                    },
                ),
                repository='origin',
                refspec='HEAD:commit_ref_name',
                option='ci.skip',
            )
        ],
    )


def test_cannot_find_git():
    with pytest.raises(RuntimeError, match='Cannot find git'):
        run_app('simple', disable_git=True)


def test_cannot_find_uv():
    with pytest.raises(RuntimeError, match='Cannot find uv'):
        run_app('simple', disable_uv=True)


def test_gitlab_no_project_name():
    with remove_gitlab_ci_vars(), pytest.raises(RuntimeError, match='CI_PROJECT_NAME'):
        run_app('simple', ci_type=ci.Type.GITLAB, disable_project_name=True)


def test_gitlab_project_name_with_env():
    with util.set_env('CI_PROJECT_NAME', 'project_name'):
        git_state = run_app(
            'simple', ci_type=ci.Type.GITLAB, disable_project_name=True, username=None
        )
    assert git_state.user_name == 'project_name-ci'


def test_gitlab_no_project_path():
    with remove_gitlab_ci_vars(), pytest.raises(RuntimeError, match='CI_PROJECT_PATH'):
        run_app('simple', ci_type=ci.Type.GITLAB, project_path=None)


def test_gitlab_no_commit_ref_name():
    with remove_gitlab_ci_vars(), pytest.raises(RuntimeError, match='CI_COMMIT_REF_NAME'):
        run_app('simple', commit_ref_name=None)


def test_gitlab_no_repository_host():
    with remove_gitlab_ci_vars(), pytest.raises(RuntimeError, match='CI_SERVER_HOST'):
        run_app('simple', repository_host=None)


def test_gitlab_no_email():
    git_state = run_app('simple', email=None)
    assert git_state.user_email == 'username@repository_host'


def test_gitlab_no_username():
    git_state = run_app('simple', username=None)
    assert git_state.user_name == 'simple-ci'


def test_gitlab_no_password():
    with pytest.raises(RuntimeError, match='password'):
        run_app('simple', password=None)



import pathlib
import typing

import tomlkit.container
import tomlkit.items
import tomlkit.toml_document

from uv_ci_tools.lib import ver


def get_path():
    return pathlib.Path('pyproject.toml')


def get_document():
    pyproject_path = get_path()
    return tomlkit.parse(pyproject_path.read_text())


def set_document(document: tomlkit.toml_document.TOMLDocument):
    get_path().write_text(document.as_string())


def get_project(document: tomlkit.toml_document.TOMLDocument):
    project_item = document['project']
    assert isinstance(project_item, dict), project_item
    return project_item


def get_project_version(project: tomlkit.container.Container):
    version_item = project['version']
    assert isinstance(version_item, str), version_item
    return ver.Version.load(version_item)


def set_project_version(project: tomlkit.container.Container, version: ver.Version):
    project['version'] = version.dump()


def get_project_name(project: tomlkit.container.Container):
    name_item = project['name']
    assert isinstance(name_item, tomlkit.items.String), name_item
    return name_item


def update_version(version_action: typing.Callable[[ver.Version], ver.Version]):
    document = get_document()
    project = get_project(document)
    old_version = get_project_version(project)
    new_version = version_action(old_version)
    set_project_version(project, new_version)
    set_document(document)
    return old_version, new_version



import os

import pytest

import test.mock.uv
import test.projects
from uv_ci_tools.app.get import app
from uv_ci_tools.lib import util

from ._util import ExecutableContext


def run_app(project_name: str, key: app.Key):
    with util.make_tmp_dir_copy(util.module_path(test.projects) / project_name) as project_dir:
        os.chdir(project_dir)
        app.get(key)


def test_simple_project_name(capsys: pytest.CaptureFixture[str]):
    run_app('simple', 'project-name')
    assert capsys.readouterr().out == 'simple'


def test_simple_project_version(capsys: pytest.CaptureFixture[str]):
    run_app('simple', 'project-version')
    assert capsys.readouterr().out == '0.1.0'


def test_simple_install_exectuable(capsys: pytest.CaptureFixture[str]):
    with ExecutableContext.make() as exe_ctx:
        exe_ctx.add_executable(test.mock.uv)

        with test.mock.uv.set_uv_tool_list_show_paths_output(
            'simple v0.0.0 (/)\n- tool (executable_path)'
        ):
            run_app('simple', 'installed-executable')
    assert capsys.readouterr().out == 'executable_path'


def test_no_tool_installed():
    with ExecutableContext.make() as exe_ctx:
        exe_ctx.add_executable(test.mock.uv)

        with (
            test.mock.uv.set_uv_tool_list_show_paths_output('simple v0.0.0 (/)'),
            pytest.raises(RuntimeError, match='No executable available'),
        ):
            run_app('simple', 'installed-executable')



import contextlib
import dataclasses
import os
import pathlib
import sys

from uv_ci_tools.lib import util


@dataclasses.dataclass
class ExecutableContext:
    dir: pathlib.Path

    @staticmethod
    @contextlib.contextmanager
    def make():
        with util.make_tmp_dir() as tmp_dir, util.set_env('PATH', tmp_dir):
            yield ExecutableContext(tmp_dir)

    def add_executable(self, script: util.HasDunderFile):
        script_path = pathlib.Path(script.__file__)
        executable_path = self.dir / script_path.stem
        executable_path.write_text(
            '\n'.join(['#!/bin/bash', f'{sys.executable} {script_path} "$@"', ''])
        )
        executable_path.chmod(0o555)
        return executable_path


@contextlib.contextmanager
def remove_gitlab_ci_vars():
    with util.use_contexts([util.set_env(var, '') for var in os.environ if var.startswith('CI_')]):
        yield



import enum

from . import gitlab
from .base import Context, PartialContext


class Type(enum.Enum):
    GITLAB = 'gitlab'

    def fill_context(self, partial: PartialContext):
        match self:
            case Type.GITLAB:  # pragma: no branch
                return gitlab.fill_context(partial)


__all__ = ['Context', 'PartialContext', 'Type']



import contextlib
import pathlib
import sys
import typing

import cyclopts
import pydantic

APP = cyclopts.App()


class Commit(pydantic.BaseModel):
    message: str
    files: dict[pathlib.Path, str]


class Pushed(pydantic.BaseModel):
    commit: Commit
    repository: str
    refspec: str
    option: str


class State(pydantic.BaseModel):
    user_name: str | None = None
    user_email: str | None = None
    remote_name_url_map: dict[str, str] = pydantic.Field(default_factory=dict)
    staged_files: dict[pathlib.Path, str] = pydantic.Field(default_factory=dict)
    commits: list[Commit] = pydantic.Field(default_factory=list)
    pushed: list[Pushed] = pydantic.Field(default_factory=list)


def get_state_path(project_dir: pathlib.Path | None):
    if project_dir is None:
        project_dir = pathlib.Path.cwd()
    return project_dir / 'git_state.json'


def get_state(project_dir: pathlib.Path | None = None):
    try:
        text = get_state_path(project_dir=project_dir).read_text()
    except FileNotFoundError:
        text = '{}'
    return State.model_validate_json(text)


def set_state(state: State, project_dir: pathlib.Path | None = None):
    get_state_path(project_dir=project_dir).write_text(state.model_dump_json())


@contextlib.contextmanager
def sync_state():
    state = get_state()
    print(state)
    yield state
    set_state(state)


@APP.command
def config(key: typing.Literal['user.name', 'user.email'], value: str) -> None:
    with sync_state() as state:
        match key:
            case 'user.name':
                state.user_name = value
            case 'user.email':
                state.user_email = value


@APP.command
def remote(action: typing.Literal['remove', 'add'], name: str, url: str | None = None):
    with sync_state() as state:
        match action:
            case 'add':
                assert name not in state.remote_name_url_map
                assert url is not None
                state.remote_name_url_map[name] = url
            case 'remove':
                del state.remote_name_url_map[name]


@APP.command
def add(*paths: pathlib.Path):
    with sync_state() as state:
        state.staged_files.update({path: path.read_text() for path in paths})


@APP.command
def commit(*, message: typing.Annotated[str, cyclopts.Parameter(name=['-m'])]):
    with sync_state() as state:
        assert len(state.staged_files) > 0
        state.commits.append(Commit(message=message, files=state.staged_files))
        state.staged_files.clear()


@APP.command
def push(
    repository: str, refspec: str, *, option: typing.Annotated[str, cyclopts.Parameter(name=['-o'])]
):
    with sync_state() as state:
        assert len(state.commits) > 0
        for commit in state.commits:
            state.pushed.append(
                Pushed(commit=commit, repository=repository, refspec=refspec, option=option)
            )
        state.commits.clear()


if __name__ == '__main__':
    APP.__call__(sys.argv[1:], print_error=True, exit_on_error=False)



import sys
import typing

from uv_ci_tools.lib import cli, pyproject, uv

APP = cli.sub_app(__name__)


def get_project():
    return pyproject.get_project(pyproject.get_document())


def get_project_name():
    return pyproject.get_project_name(get_project())


def get_project_version():
    return pyproject.get_project_version(get_project())


type Key = typing.Literal['project-version', 'project-name', 'installed-executable']


@APP.default
def get(key: Key, /):
    match key:
        case 'project-version':
            text = get_project_version().dump()
        case 'project-name':
            text = get_project_name()
        case 'installed-executable':  # pragma: no branch
            project_name = get_project_name()
            installed_package = uv.get_installed_package(project_name)
            first_tool = next(iter(installed_package.tools), None)
            if first_tool is None:
                msg = f'No executable available for {project_name}'
                raise RuntimeError(msg)
            text = str(first_tool.path)

    sys.stdout.write(text)



import pytest

import test.mock.python
import test.mock.uv
from uv_ci_tools.app.pre_compile import app
from uv_ci_tools.lib import ci, util

from ._util import ExecutableContext


def run_app(ci_type: ci.Type = ci.Type.GITLAB, project_name: str | None = 'project_name'):
    with (
        util.set_env('CI_PROJECT_PATH', 'project_path'),
        util.set_env('CI_COMMIT_REF_NAME', 'commit_ref_name'),
        util.set_env('CI_SERVER_HOST', 'server_host'),
    ):
        app.pre_compile(ci_type=ci_type, project_name=project_name)


def test_simple(capsys: pytest.CaptureFixture[str]):
    with util.make_tmp_dir() as install_path, ExecutableContext.make() as exe_ctx:
        exe_ctx.add_executable(test.mock.uv)
        python_executable = exe_ctx.add_executable(test.mock.python)

        bin_install_dir = install_path / 'bin'
        bin_install_dir.mkdir()
        installed_python_executable = bin_install_dir / 'python'
        installed_python_executable.symlink_to(python_executable)

        with test.mock.uv.set_uv_tool_list_show_paths_output(
            f'project_name v0.0.0 ({install_path})'
        ):
            run_app()
    assert 'compileall' not in capsys.readouterr().out


def test_not_installed():
    with ExecutableContext.make() as exe_ctx:
        exe_ctx.add_executable(test.mock.uv)

        with (
            test.mock.uv.set_uv_tool_list_show_paths_output(''),
            pytest.raises(RuntimeError, match='Cannot find installed package'),
        ):
            run_app()


def test_no_python_executable():
    with util.make_tmp_dir() as install_path, ExecutableContext.make() as exe_ctx:
        exe_ctx.add_executable(test.mock.uv)

        with (
            test.mock.uv.set_uv_tool_list_show_paths_output(
                f'project_name v0.0.0 ({install_path})'
            ),
            pytest.raises(RuntimeError, match='Canont find python executable'),
        ):
            run_app()



import dataclasses


@dataclasses.dataclass
class PartialContext:
    project_name: str | None = None
    project_path: str | None = None
    commit_ref_name: str | None = None
    email: str | None = None
    username: str | None = None
    password: str | None = None
    repository_host: str | None = None


@dataclasses.dataclass
class Context:
    project_name: str
    project_path: str
    commit_ref_name: str
    email: str
    username: str
    password: str | None
    repository_host: str



import contextlib
import dataclasses
import functools
import itertools
import pathlib
import subprocess

from uv_ci_tools.lib import util, ver


def get_lock_file_path():
    return pathlib.Path('uv.lock')


def update_lock_file(
    package_name: str, old_version: ver.Version, new_version: ver.Version, *, no_uv: bool
) -> None:
    if not no_uv:
        uv_exe_path = util.get_exe_path('uv')
        subprocess.run([uv_exe_path, 'lock'], check=True)
        return

    lock_file_path = get_lock_file_path()

    try:
        old_lock_lines = lock_file_path.read_text().splitlines()
    except FileNotFoundError:
        lock_file_path.write_text('')
        return

    if len(old_lock_lines) == 0:
        return

    new_lock_lines = [old_lock_lines[0]]

    line1_target = f'name = "{package_name}"'
    line2_target = f'version = "{old_version.dump()}"'

    for line1, line2 in itertools.pairwise(old_lock_lines):
        if line1 == line1_target and line2 == line2_target:
            new_lock_lines.append(f'version = "{new_version.dump()}"')
        else:
            new_lock_lines.append(line2)

    new_lock_lines.append('')

    lock_file_path.write_text('\n'.join(new_lock_lines))


@dataclasses.dataclass
class InstalledTool:
    name: str
    path: pathlib.Path


@dataclasses.dataclass
class InstalledPackage:
    name: str
    version: ver.Version
    path: pathlib.Path
    tools: list[InstalledTool]

    @functools.cached_property
    def python_executable(self):
        python_executable = self.path / 'bin' / 'python'
        if not util.is_executable(python_executable):
            msg = f'Canont find python executable for package {self.name}'
            raise RuntimeError(msg)
        return python_executable


def list_installed_packages():
    uv_exe_path = util.get_exe_path('uv')
    process = subprocess.run(
        [uv_exe_path, 'tool', 'list', '--show-paths'], check=True, capture_output=True, text=True
    )
    assert process.stdout is not None
    packages: list[InstalledPackage] = []
    for line in process.stdout.splitlines():
        match line.split():
            case ['-', name, path_part]:
                last_package = next(reversed(packages), None)
                if last_package is not None:
                    path = pathlib.Path(path_part[1:-1])
                    last_package.tools.append(InstalledTool(name, path))
            case [name, version_part, path_part]:
                with contextlib.suppress(RuntimeError):
                    version = ver.Version.load(version_part[1:])
                    path = pathlib.Path(path_part[1:-1])
                    packages.append(InstalledPackage(name, version, path, tools=[]))
            case _:
                pass
    return packages


def get_installed_package(project_name: str):
    project_name_installed_package_map = {
        package.name: package for package in list_installed_packages()
    }
    installed_package = project_name_installed_package_map.get(project_name)
    if installed_package is None:
        msg = f'Cannot find installed package for {project_name}'
        raise RuntimeError(msg)
    return installed_package



"""Test the CLI entrypoint, argument parsing and error printing."""

import contextlib
import dataclasses
import sys

import pytest

import uv_ci_tools
from uv_ci_tools import __main__

type _Capsys = pytest.CaptureFixture[str]


@dataclasses.dataclass
class _Console:
    out: str
    err: str


@contextlib.contextmanager
def _setup(argv: list[object], capsys: _Capsys):
    previous_argv = sys.argv
    sys.argv = [sys.executable, *(map(str, argv))]
    console = _Console(out='', err='')
    capsys.readouterr()  # clears buffer
    try:
        yield console
    finally:
        capture = capsys.readouterr()
        console.out = capture.out
        console.err = capture.err
        sys.argv = previous_argv


def test_main_version(capsys: _Capsys) -> None:
    with _setup(['--version'], capsys) as console:
        __main__.main()

    assert console.out == uv_ci_tools.__version__ + '\n'



from uv_ci_tools.lib import util

from . import base


def fill_context(partial: base.PartialContext):
    if partial.project_name is None:
        partial.project_name = util.get_env_var('CI_PROJECT_NAME')
    if partial.project_path is None:
        partial.project_path = util.get_env_var('CI_PROJECT_PATH')
    if partial.commit_ref_name is None:
        partial.commit_ref_name = util.get_env_var('CI_COMMIT_REF_NAME')
    if partial.repository_host is None:
        partial.repository_host = util.get_env_var('CI_SERVER_HOST')
    if partial.username is None:
        partial.username = f'{partial.project_name}-ci'
    if partial.email is None:
        partial.email = f'{partial.username}@{partial.repository_host}'
    return base.Context(
        project_name=partial.project_name,
        project_path=partial.project_path,
        commit_ref_name=partial.commit_ref_name,
        username=partial.username,
        password=partial.password,
        repository_host=partial.repository_host,
        email=partial.email,
    )



def _app(*, name: str | None):
    import cyclopts

    import uv_ci_tools

    app = cyclopts.App(name=name, version=uv_ci_tools.__version__)
    app['--help'].group = 'Extra'
    app['--version'].group = 'Extra'
    return app


def _format_command_name(name: str):
    return name.split('.')[-2].replace('_', '-')


def main_app():
    return _app(name=None)


def sub_app(name: str):
    return _app(name=_format_command_name(name))



import pathlib
import typing

import cyclopts

APP = cyclopts.App()


@APP.command
def commit(
    _: typing.Annotated[typing.Literal['compileall'], cyclopts.Parameter(name=['-m'])],
    directory: pathlib.Path,
):
    print('compileall')
    (directory / '__pycache__').mkdir()



import os

from uv_ci_tools.lib import util


def test_set_env():
    name = 'MY_VARIABLE_687043857'
    assert os.getenv(name) is None
    with util.set_env(name, value=None):
        assert os.getenv(name) is None
    with util.set_env(name, value='VALUE'):
        assert os.getenv(name) == 'VALUE'
        with util.set_env(name, value=None):
            assert os.getenv(name) is None
        assert os.getenv(name) == 'VALUE'
    assert os.getenv(name) is None



import dataclasses
import enum
import typing


class IncrementKind(enum.Enum):
    MAJOR = 'major'
    MINOR = 'minor'
    PATCH = 'patch'


@dataclasses.dataclass(frozen=True)
class Version:
    major: int
    minor: int
    patch: int

    @classmethod
    def load(cls, value: str) -> typing.Self:
        if not value:
            msg = 'no value'
            raise RuntimeError(msg)

        parts = value.split('.')
        if len(parts) != len(dataclasses.fields(cls)):
            msg = f'should be 3 parts separated by dots: {value}'
            raise RuntimeError(msg)

        parts_int: list[int] = []
        for part in parts:
            try:
                parts_int.append(int(part))
            except ValueError as e:
                msg = f'parts should be integers: {value}'
                raise RuntimeError(msg) from e

        return cls(*parts_int)

    def dump(self) -> str:
        return f'{self.major}.{self.minor}.{self.patch}'

    def incremented(self, kind: IncrementKind) -> typing.Self:
        match kind:
            case IncrementKind.MAJOR:
                return self.__class__(major=self.major + 1, minor=0, patch=0)
            case IncrementKind.MINOR:
                return self.__class__(major=self.major, minor=self.minor + 1, patch=0)
            case _:
                return self.__class__(major=self.major, minor=self.minor, patch=self.patch + 1)



import base64
import contextlib
import os
import pathlib
import sys
import typing

import cyclopts

APP = cyclopts.App(version='0.1.0')


@APP.command
def lock() -> None:
    pathlib.Path('uv.lock').write_text('')


_UV_TOOL_LIST_SHOW_PATHS_B64_VAR = 'UV_TOOL_LIST_SHOW_PATHS_B64_VAR'


@contextlib.contextmanager
def set_uv_tool_list_show_paths_output(output: str):
    from uv_ci_tools.lib import util

    with util.set_env(_UV_TOOL_LIST_SHOW_PATHS_B64_VAR, base64.b64encode(output.encode()).decode()):
        yield


@APP.command
def tool(_: typing.Literal['list'], /, *, show_paths: bool = False) -> None:
    if show_paths:
        print(base64.b64decode(os.getenv(_UV_TOOL_LIST_SHOW_PATHS_B64_VAR, '')).decode())


if __name__ == '__main__':
    APP.__call__(sys.argv[1:], print_error=True, exit_on_error=False)



import pathlib
import subprocess

from uv_ci_tools.lib import ci, util


def get_git():
    git_exe_path = util.get_exe_path('git')

    def git(*args: util.StrOrBytesPath, no_check: bool = False):
        subprocess.run([git_exe_path, *args], check=not no_check)

    return git


def push(*paths: pathlib.Path, message: str, ci_ctx: ci.Context):
    if ci_ctx.password is None:
        msg = 'Cannot git push without password'
        raise RuntimeError(msg)

    git = get_git()
    origin_name = 'origin'
    origin_url = (
        f'https://{ci_ctx.username}:{ci_ctx.password}'
        f'@{ci_ctx.repository_host}/{ci_ctx.project_path}.git'
    )
    git('config', 'user.name', ci_ctx.username)
    git('config', 'user.email', ci_ctx.email)
    git('remote', 'remove', origin_name, no_check=True)
    git('remote', 'add', origin_name, origin_url)
    git('add', *paths)
    git('commit', '-m', message)
    git('push', origin_name, f'HEAD:{ci_ctx.commit_ref_name}', '-o', 'ci.skip')



import sys

import cyclopts
import pytest

import uv_ci_tools

APP = cyclopts.App(version='0.0.0')


@APP.default
def main(*, expect_full: bool = False, verbosity: int = 0):
    args = [
        f'--cov={uv_ci_tools.__name__}',
        '--cov-branch',
        '--cov-report=html',
        '--numprocesses=auto',
        f'--verbosity={verbosity}',
    ]
    if expect_full:
        args.append('--cov-fail-under=100')

    return pytest.main(args)


if __name__ == '__main__':
    APP.__call__(sys.argv[1:], print_error=True, exit_on_error=False)



from uv_ci_tools.lib import ci, cli, git, pyproject, uv, ver

APP = cli.sub_app(__name__)


@APP.default
def version(
    increment_kind: ver.IncrementKind,
    /,
    *,
    ci_type: ci.Type = ci.Type.GITLAB,
    project_name: str | None = None,
    project_path: str | None = None,
    commit_ref_name: str | None = None,
    email: str | None = None,
    username: str | None = None,
    password: str | None = None,
    repository_host: str | None = None,
    no_uv: bool | None = None,
):
    ci_ctx = ci_type.fill_context(
        ci.PartialContext(
            project_name=project_name,
            project_path=project_path,
            commit_ref_name=commit_ref_name,
            email=email,
            username=username,
            password=password,
            repository_host=repository_host,
        )
    )
    old_version, new_version = pyproject.update_version(
        lambda version: version.incremented(increment_kind)
    )
    uv.update_lock_file(
        pyproject.get_project_name(pyproject.get_project(pyproject.get_document())),
        old_version=old_version,
        new_version=new_version,
        no_uv=no_uv or False,
    )
    git.push(
        pyproject.get_path(),
        uv.get_lock_file_path(),
        message=f'Bumped version to {new_version.dump()}',
        ci_ctx=ci_ctx,
    )



import contextlib
import os
import pathlib
import shutil
import subprocess
import tempfile
import typing


def get_exe_path(name: str):
    exe_path = shutil.which(name)
    if exe_path is None:
        msg = f'Cannot find {name} executable'
        raise RuntimeError(msg)
    subprocess.run([exe_path, '--version'], capture_output=True, check=True)
    return pathlib.Path(exe_path)


def get_env_var(name: str):
    value = os.getenv(name, '')
    if len(value) == 0:
        msg = f'{name} is not set'
        raise RuntimeError(msg)
    return value


class HasDunderFile(typing.Protocol):
    @property
    def __file__(self) -> str: ...


def module_path(module: HasDunderFile):
    return pathlib.Path(module.__file__).parent.absolute()


@contextlib.contextmanager
def make_tmp_dir(base: pathlib.Path | None = None):
    with tempfile.TemporaryDirectory(dir=base) as tmp_dir:
        yield pathlib.Path(tmp_dir).absolute()


@contextlib.contextmanager
def make_tmp_dir_copy(dir_path: pathlib.Path):
    with make_tmp_dir(dir_path.parent) as tmp_dir:
        tmp_dir_copy_path = tmp_dir / dir_path.name
        shutil.copytree(dir_path, tmp_dir_copy_path)
        yield tmp_dir_copy_path


@contextlib.contextmanager
def set_env(name: str, value: object):
    old_value = os.getenv(name)
    if value is not None:
        os.environ[name] = str(value)
    elif old_value is not None:
        del os.environ[name]
    try:
        yield
    finally:
        if old_value is not None:
            os.environ[name] = old_value
        elif value is not None:
            del os.environ[name]


class PathLike[T](typing.Protocol):
    def __fspath__(self) -> T: ...


type StrOrBytesPath = str | bytes | PathLike[str] | PathLike[bytes]


def is_file(path: pathlib.Path):
    return path.exists() and path.is_file()


def is_executable(path: pathlib.Path):
    return is_file(path) and os.access(path, os.X_OK)


@contextlib.contextmanager
def devnull():
    with pathlib.Path(os.devnull).open(mode='w') as f:
        yield f


@contextlib.contextmanager
def use_contexts[T](
    contexts: list[contextlib.AbstractContextManager[T]],
) -> typing.Generator[list[T], None, None]:
    with contextlib.ExitStack() as stack:
        yield [stack.enter_context(cm) for cm in contexts]
